# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Template.pm,v 1.21 2001/10/16 00:30:22 btrott Exp $

package MT::Template;
use strict;

use vars qw( %TYPES );
%TYPES = (
    'index' => 'Index',
    archive => 'Archive',
    category => 'Category Archive',
    individual => 'Individual',
    comments => 'Comment Listing',
    comment_preview => 'Comment Preview',
    custom => 'Custom',
);

use base qw( MT::Object );
__PACKAGE__->install_properties({
    columns => [
        'id', 'blog_id', 'name', 'type', 'outfile', 'text'
    ],
    indexes => {
        blog_id => 1,
        name => 1,
        type => 1,
    },
    datasource => 'template',
    primary_key => 'id',
});

use MT::Builder;
use MT::Blog;
use MT::ConfigMgr;

sub build {
    my $tmpl = shift;
    my($ctx, $cond) = @_;
    my $text = $tmpl->text;
    my $build = MT::Builder->new;
    my $blog_id = $tmpl->blog_id;
    $ctx->stash('blog_id', $blog_id);
    my $blog = MT::Blog->load($blog_id) or
        return $tmpl->error("Load of blog '$blog_id' failed: " .
                            MT::Blog->errstr);
    $ctx->stash('blog', $blog);
    MT::ConfigMgr->instance->TimeOffset($blog->server_offset);
    my $tokens = $build->compile($ctx, $text) or
        return $tmpl->error("Parse error in template '" . $tmpl->name .
                            "': " . $build->errstr);
    defined(my $res = $build->build($ctx, $tokens, $cond)) or
        return $tmpl->error("Build error in template '" . $tmpl->name .
                            "': " . $build->errstr);
    $res;
}

1;
