#!/usr/bin/perl -w

# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: mt-import.cgi,v 1.11 2001/10/22 03:42:07 btrott Exp $
use strict;

local $|=1;
print "Content-Type: text/html\n\n";

my($MT_DIR);
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    print "<pre>\n";

    require MT::App::CMS;
    require MT::Entry;
    require MT::Author;
    require MT::Permission;
    require File::Spec;
    require CGI;

    my $mt = MT::App::CMS->new( Config => $MT_DIR . 'mt.cfg' );

    my($author, $first_time) = $mt->login;
    unless ($author) {
        die "You are not logged in. Please log in through the main\n" .
            "Movable Type system, then import your files through there.\n";
    }

    my $q = $mt->{query};
    my $SEP = ('-' x 8);

    my %authors;
    require MT::Blog;
    my $blog_id = $q->param('blog_id') or die "No blog ID";
    my $blog = MT::Blog->load($blog_id)
        or die "Blog load failed: ", MT::Blog->errstr;
    my $author_id = $author->id;
    my $perms = MT::Permission->load({ author_id => $author_id,
                                       blog_id => $blog_id });
    die "You do not have import permissions"
        unless $perms && $perms->can_edit_config;
    print "Importing entries into blog '", $blog->name, "'\n";
    my($pass);
    my $import_as_me = $q->param('import_as_me');
    if ($import_as_me) {
        print "Importing entries as author '", $author->name, "'\n";
    } else {
        $pass = $q->param('password')
            or die "You need to provide a password if you are going to " .
                   "create new authors for each author listed in your blog.";
        print "Creating new authors for each author found in the blog\n";
    }
    print "\n";

    my $t_start = $q->param('title_start');
    my $t_end = $q->param('title_end');
    my $allow_comments = $blog->allow_comments_default ? 1 : 0;

    my $dir = $MT_DIR . 'import';
    opendir DH, $dir or die "Can't open directory '$dir': $!";
    for my $f (readdir DH) {
        next if $f =~ /^\./;
        print "Importing entries from file '$f'\n";
        my $file = File::Spec->catfile($dir, $f);
        open FH, $file or die "Can't open file '$file': $!";
        local $/ = $SEP;
        while (<FH>) {
            my($meta, $body) = $_ =~ m!^(.*?)\r?\nSTART BODY:\r?\n(.*)$!s;
            $body =~ s!\s*$SEP\s*$!!;
            next unless $meta && $body;
            my $entry = MT::Entry->new;
            $entry->blog_id($blog_id);
            $entry->status(MT::Entry::RELEASE());
            $entry->allow_comments($allow_comments);
            $entry->author_id($author_id) if $import_as_me;
            for my $line (split /\r?\n/, $meta) {
                next unless $line;
                my($key, $val) = split /\s*:\s*/, $line, 2;
                if ($key eq 'AUTHOR' && !$import_as_me) {
                    my $author;
                    unless ($author = $authors{$val}) {
                        $author = MT::Author->new;
                        $author->name($val);
                        $author->set_password($pass);
                        print "Creating new author ('$val')...";
                        if ($author->save) {
                            print "ok\n";
                        } else {
                            print "failed\n";
                            die "Author save failed: ", $author->errstr;
                        }
                        $authors{$val} = $author;
                        print "Assigning permissions for new author...";
                        my $perms = MT::Permission->new;
                        $perms->blog_id($blog_id);
                        $perms->author_id($author->id);
                        $perms->can_post(1);
                        if ($perms->save) {
                            print "ok\n";
                        } else {
                            print "failed\n";
                            die "Permission save failed: ", $perms->errstr;
                        }
                    }
                    $entry->author_id($author->id);
                } elsif ($key eq 'TITLE') {
                    $entry->title($val);
                } elsif ($key eq 'DATE') {
                    my($mo, $d, $y, $h, $m, $s, $ampm) = $val =~
                    m!^(\d{1,2})/(\d{1,2})/(\d{2,4}) (\d{1,2}):(\d{1,2}):(\d{1,2}) (\w{2})$! or
                    die "Invalid date format '$val'; must be MM/DD/YYYY HH:MM:SS AM|PM\n";
                    if ($ampm eq 'PM' && $h < 12) {
                        $h += 12;
                    } elsif ($ampm eq 'AM' && $h == 12) {
                        $h = 0;
                    }
                    if (length($y) == 2) {
                        $y += 1900;
                    }
                    my $ts = sprintf "%04d%02d%02d%02d%02d%02d",
                        $y, $mo, $d, $h, $m, $s;
                    $entry->created_on($ts);
                }
            }
            unless ($entry->title) {
                if ($t_start && $t_end && $body =~
                    s!\Q$t_start\E(.*?)\Q$t_end\E\s*!!) {
                    $entry->title($1);
                } else {
                    (my $title = $body) =~ s!<[^>]+>!!g;
                    my @words = split /\s+/, $title;
                    my $max = @words > 5 ? 5 : @words;
                    $entry->title( join ' ', @words[0..$max-1] );
                }
            }
            $entry->text($body);
            print "Creating new entry ('", $entry->title, "')... ";
            if ($entry->save) {
                print "ok (ID ", $entry->id, ")\n";
            } else {
                print "failed\n";
                die "Entry save failed: ", $entry->errstr;
            }
        }
    }
    closedir DH;
};
if ($@) {
    print "\nAn error occurred while importing your data:\n$@";
} else {
    print "\nAll data imported successfully! Make sure that you remove\n" .
          "the files that you imported from the 'import' folder, so that\n" .
          "if/when you run mt-import.cgi again, those files will not be\n" .
          "re-imported.\n";
}

print "</pre>\n";
